# Filename: andy3s.awk
#
# Last update: Mon Mar  3 20:31:53 2003 by Rob MacLeod
#    - added in support for endocardial sock leads
# Last update: Wed May 15 20:43:35 2002 by Rob MacLeod
#    - added case of more than 512 channel sock or cage
# Last update: Fri Dec 29 23:05:31 2000 by Rob MacLeod
#    - created
#
# Make the mux mapping file for the combo of
# Bank #1: 192 lead Andy3 tank
# Banks #2-4: second andy 3 bank, sock (if present), needles (if present), 
#        endocardial leads (if present) grounded leads 
#
# Usage: awk -f andy3.awk numtankleads=T  numsockleads=S numendoleads=E
#               numneedles=N full=1/0 tempfile
#
# where   
#         numtankleads = number of andy3 leads, either 192 or 370
#         numsockleads = the number of sock leads in bank 2
#         numendoleads = the number of endocardial leads at the end of bank 2
#         numneedles = the number of needles to follow the sock
#         full = 1 for a full mux mapping file for the acquistion program
#              = 0 for mux mapping of just the useful leads.
#	  tempfile = file name of a file that contains the mux mode value
#                      (single = 512 channels or double = 1024 channels)
#
# NOTE: we assume that we have all the tank leads in one bank and that 
#       anything that follows will be in subsequent banks.  We also 
#       assume that for the 512 channels setup, the sock will have 
#       128 leads or at least some number of leads that allows the needles
#       to follow directly after the last sock lead.
# NOTE2: so far we cannot handle more than 192 andy3 leads
# 
######################################################################
BEGIN{
    numperneedle=10;
    numchans=0;
    numneedlechans = 0;
    numsockchans = 0;
    numendochans = 0;
    numfillchans = 0;
    bankoffset[1] = 1;
    bankoffset[2] = 3;
    bankoffset[3] = 2;
    bankoffset[4] = 4;
    bankfill[1] = 0;
    bankfill[2] = 0;
    bankfill[3] = 0;
    bankfill[4] = 0;
}
/single/{
# Error checking
    if ( numtankleads + numsockleads + numendoleads + \
	 numneedles * numperneedle > 512 ) {
	printf(" Error: we cannot fit %d leads in a 512 MUX\n",
	       numtankleads + numsockleads + numneedles * numperneedle)\
	    > "/dev/tty";
	exit -1;
    } else if ( numtankleads < 192 ) {
	printf(" Error: We expect at least 192 tank leads not %d\n",
	       numtankleads)\
	    > "/dev/tty";
	exit -1;
    }

# First the 192 lead andy3 tank: 1, 3, 5, 7, 9...
    if ( full == 1 ) {
	printf("512 channels\n");
    } else {
	printf("%d channels\n", \
	       numtankleads + numsockleads + numneedles * numperneedle);
    }
    for (lnum=0; lnum < 192; lnum++) {
	printf("%4d ", lnum*2 + 1);
	numchans++;
	if ( numchans % 8 == 0 ){
	    printf("\n");
	}
    }
    printf("Wrote %d channels of tanks\n", numchans) > "/dev/tty";

# If we have the second bank of andy 3 leads: 2, 4, 6, 8, ...

    if ( numtankleads > 192 ) {
# Put the first connectors worth (48 leads) at the end of bank 1.
	for (lnum=0; lnum < 48; lnum++) {
	    printf("%4d ", (192 + lnum) * 2 + 1);
	    numchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
# Put the rest in the beginnging of the second bank.
	for (lnum=192+48; lnum < numtankleads-192; lnum++) {
	    printf("%4d ", lnum*2 + 2);
	    numchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
    printf("Wrote %d additional channels of tanks for a total of %d\n",\
	   numtankleads - 192, numchans) > "/dev/tty";

    } else if ( numsockleads > 0 ) {

# Now the sock if we have one: 2, 4, 6, 8, 10, ...
	for (lnum=0; lnum < numsockleads; lnum++) {
	    printf("%4d ", lnum*2 + 2);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	printf("Wrote %d channels of sock\n", numsockchans) > "/dev/tty";
    }

# Now the needles: after the others, in the bank 2, 4, 6, 8, 10, ...
    for (nnum=0; nnum < numneedles; nnum++) {
	for (elnum=0; elnum < numperneedle; elnum++) {
	    printf("%4d ", \
		   (numtankleads-192-48 + numsockleads + \
		    nnum * numperneedle + elnum) * 2 + 2);
	    numchans++;
	    numneedlechans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
    }
    printf("Wrote %d channels of needles\n", numneedlechans) > "/dev/tty";

# Now the fill at the end of the file to make up the full 512 channels

    if ( full == 1 ) {
	for (lnum=192+48; lnum < 256; lnum++) {
	    printf("%4d ", lnum*2 + 1);
	    numchans++;
	    numfillchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	for (lnum=numtankleads-192-48+numsockleads+numneedles; \
		 lnum < 256; lnum++) {
	    printf("%4d ", lnum*2 + 2);
	    numchans++;
	    numfillchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	printf("Wrote %d channels of end fill\n", numfillchans) > "/dev/tty";
    }

    printf("For a total of %d channels\n", numchans) > "/dev/tty";
    printf("\n");
}
/double/{

# Error checking
    if ( numtankleads + numsockleads + numendoleads + \
	 numneedles * numperneedle > 1024 ) {
	printf(" Error: we cannot fit %d leads in a 1024 MUX\n",
	       numtankleads + numsockleads + numendoleads + \
	       numneedles * numperneedle) > "/dev/tty";
	exit -1;
    } else if ( numtankleads < 192 ) {
	printf(" Error: We expect at least 192 tank leads not %d\n",
	       numtankleads)\
	    > "/dev/tty";
	exit -1;
    }

# First the andy 3 tank (192 leads): 1, 5, 9, 13, 17....
    if ( full == 1 ) {
	printf("1024 channels\n");
    } else {
	printf("%d channels\n", \
	       numtankleads + numsockleads + numendoleads + \
	       numneedles * numperneedle);
    }
    for (lnum=0; lnum < 192; lnum++) {
	printf("%4d ", lnum*4 + 1);
	numchans++;
	if ( numchans % 8 == 0 ){
	    printf("\n");
	}
    }
    printf("Wrote %d channels of tank\n", numchans) > "/dev/tty";
    bankfill[1] = 192;

# Next any additional tank points:

    if ( numtankleads > 192 ) {
	nextbankoffset = 2;
# Put the next 48 leads in the first bank
	for (lnum=0; lnum < 48; lnum++) {
	    printf("%4d ", (192 + lnum) * 4 + 1);
	    numchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	printf("Wrote 48 channels of secondary tank in bank 1\n")\
	    > "/dev/tty";
	bankfill[1] = bankfill[1] + 48;

# Now move to the second bank of the first MUX
	for (lnum=0; lnum < numtankleads-192-48; lnum++) {
	    printf("%4d ", lnum*4 + 3);
	    numchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}

	bankfill[2] = numtankleads - 192 - 48;
	printf("Wrote %d channels of secondary tank in bank 2\n", 
	       numtankleads - 192 - 48)\
	    > "/dev/tty";

# Now add in the endocardial sock electrodes

	if ( numendoleads > 0 && numendoleads < 512 - numtankleads ) {
# ?? Not clear what value we should have here
#	    firstendochan = numtankleads - 192 - 48;
	    firstendochan = 384 - 192 - 48;
	    for (lnum=0; lnum < numendoleads; lnum++) {
		printf("%4d ", (firstendochan + lnum)*4 + 3);
		numchans++;
		if ( numchans % 8 == 0 ){
		    printf("\n");
		}
	    }
	    bankfill[2] = numtankleads - 192 - 48 + numtankleads;
	    printf("Wrote %d channels of endo sock in bank 2\n", 
		   numendoleads)\
		> "/dev/tty";
	    
	}
	
    } else {
	nextbankoffset=3;
    }

# Now for the socks.
#      we have different strategies for each sock type:
#      For a 128 lead sock, we use bank 2: 3, 7, 11, 15, ... 1023
#      For the 490 sock, we need two banks and split the sock into
#          two halves: 240 and 250
#      For the cage, which looks like a 600 channel sock, we set up as 
#      follows:
#        Master Bank 1: leads 193--240, connector #1 (1--48)
#                       leads 241--256, empty
#        Master Bank 2: leads 1-240, connectors #2--6 (49--288)
#                       leads 241--256, empty
#        Slave  Bank 1: leads 1-240, connectors #7--11 (289--528)
#                       leads 241--256, empty
#        Slave  Bank 2: leads 1-82, connectors #12--13 (529--610)
#                       leads 83--256, empty

    if ( numsockleads == 128 ) {
	for (lnum=0; lnum < numsockleads; lnum++) {
	    printf("%4d ", lnum*4 + nextbankoffset);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	if ( nextbankoffset == 3) {
	    bankfill[2] =  128;
	    nextbankoffset = 2;
	} else {
	    nextbankoffset = 4;
	    bankfill[3] =  128
	}

    } else if ( numsockleads == 490 ) {
	printf("We have a 490 sock\n") > "/dev/tty";

	for (lnum=0; lnum < 240; lnum++) {
	    printf("%4d ", lnum*4 + nextbankoffset);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	if ( nextbankoffset == 3) {
	    nextbankoffset = 2;
	    bankfill[2] =  240;
	} else if ( nextbankoffset == 2) {
	    nextbankoffset = 4;
	    bankfill[3] =  240;
	} else {
	    printf("\n*** Not enough space for the full 490 lead sock!\n") > \
		"/dev/tty";
	    exit -1;
	}
	for (lnum=0; lnum < 250; lnum++) {
	    printf("%4d ", lnum*4 + nextbankoffset);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	if ( nextbankoffset == 2) {
	    nextbankoffset = 4;
	    bankfill[3] =  250;
	} else {
	    bankfill[4] =  250;
	    nextbankoffset = -1;
	}

# Socks or cage with more than 512 leads.

    } else if ( numsockleads > 512 ) {
	printf("We have a big sock or cage with %d leads\n", numsockleads) > "/dev/tty";

# Set this variable to 1 if we leave a gap at the end of the first bank.

	qleavegap=1;

# First, the case when we have a gap now, we start the cage/sock at the 
# beginning of the second bank.

	if ( qleavegap ) {
	    bankfill[1]=192;
	    numsockinbank1=0;
	     printf("Assume we leave the end of bank 1 open.\n") > "/dev/tty";
	} else {

# This is the code for the case in which cage/sock follows tank in the same
# bank, so leads 193--240.
	    printf("Assume we use populate the  end of bank 1.\n") > "/dev/tty";
	    for (lnum=0; lnum < 48; lnum++) {
		printf("%4d ", (192 + lnum) * 4 + 1);
		numchans++;
		numsockchans++;
		if ( numchans % 8 == 0 ){
		    printf("\n");
		}
	    }
	    bankfill[1] =  240;
	    numsockinbank1=48;
	}

	for (lnum=0; lnum < 240; lnum++) {
	    printf("%4d ", lnum*4 + 3);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	bankfill[2] =  240;
	numleft = numsockleads - (480 + numsockinbank1);
	if ( numleft < 0 ) {
	    numinbank3 = numsockleads - (240 + numsockinbank1);
	    bankfill[4] =  0;
	    printf("We can fit everything into three banks\n") > "/dev/tty";
	    printf("We have %d left for bank 3\n", numinbank3) > "/dev/tty";
	} else {
	    numinbank3 = 240;
	    bankfill[4] =  numleft;
	    printf("We need all four banks\n") > "/dev/tty";
	    printf("with %d in the last bank\n", numleft) > "/dev/tty";
	}
	bankfill[3] =  numinbank3;

	for (lnum=0; lnum < numinbank3; lnum++) {
	    printf("%4d ", lnum*4 + 2);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	for (lnum=0; lnum < numleft; lnum++) {
	    printf("%4d ", lnum*4 + 4);
	    numchans++;
	    numsockchans++;
	    if ( numchans % 8 == 0 ){
		printf("\n");
	    }
	}
	

    } else {
	printf(" Sorry, we have no code for a sock with %d electrodes"\
	       "\n Please talk to Rob or Bonnie\n", numsockleads) > "/dev/tty";
    }
    printf("Wrote %d channels of sock\n", numsockchans) > "/dev/tty";

# Now the needles: 2, 6, 10, 14, ...1022, and/or 4, 8, 12, ... 1024
# We assume the needles, where they come, start a new bank.

    if ( numneedles > 0 ) {
	if ( nextbankoffset > 0 ) {
	    if ( numneedles * numperneedle <=256 ) {
		numneedles1 = numneedles;
		numneedles2 = 0;
	    } else {
		numneedles1 = 25;
		numneedles2 = numneedles - 25;
	    }
	    for (nnum=0; nnum < numneedles1; nnum++) {
		for (elnum=0; elnum < numperneedle; elnum++) {
		    printf("%4d ", \
			   ( nnum * numperneedle + elnum) * 4 + nextbankoffset);
		    numchans++;
		    numneedlechans++;
		    if ( numchans % 8 == 0 ){
			printf("\n");
		    }
		}
	    }
	    if ( nextbankoffset == 3) {
		nextbankoffset = 2;
		bankfill[2] =  numneedles1 * numperneedle;
	    } else if ( nextbankoffset == 2) {
		nextbankoffset = 4;
		bankfill[3] =  numneedles1 * numperneedle;
	    } else if ( numneedles2 > 0 ) {
		printf("\n*** Not enough space for all the needles!\n") > \
		    "/dev/tty";
		exit -1;
	    }
	    for (nnum=0; nnum < numneedles2; nnum++) {
		for (elnum=0; elnum < numperneedle; elnum++) {
		    printf("%4d ", \
			   ( nnum * numperneedle + elnum) * 4 + nextbankoffset);
		    numchans++;
		    numneedlechans++;
		    if ( numchans % 8 == 0 ){
			printf("\n");
		    }
		}
	    }
	    printf("Wrote %d channels of needles\n", numneedlechans) \
		> "/dev/tty";
	}
    }

# Now the fill at the end of the file to make up the full 1024 channels

    if ( full == 1 ) {
	printf("Filling up space from the empty parts of banks\n") \
	    > "/dev/tty";
	printf("Bankfills are %d, %d, %d, and %d\n", \
	   bankfill[1], bankfill[2], bankfill[3], bankfill[4]) > "/dev/tty";
	for (banknum=1; banknum <= 4; banknum++) {
	    for (lnum=bankfill[banknum]; lnum < 256; lnum++) {
		printf("%4d ", lnum*4 + bankoffset[banknum]);
		numchans++;
		numfillchans++;
		if ( numchans % 8 == 0 ){
		    printf("\n");
		}
	    }
	}
	printf("Wrote %d channels of end fill\n", numfillchans) > "/dev/tty";
    }
    
    printf("For a total of %d channels\n", numchans) > "/dev/tty";
    printf("\n");

}
